<?php

class Op5reports_options extends Report_options
{
	public static $right_sub_type = false;
	protected $subs = array();

	public function __construct($options=false)
	{
		$this->properties = array(
			'use_pnp' => array('type' => 'bool', 'default' => false),
			'use_summary' => array('type' => 'bool', 'default' => false),
		);
		parent::__construct($options);
		if (is_a($options, 'Op5reports_options')) {
			foreach ($options->subs as $sub) {
				$class = get_class($sub);
				$type = substr($class, 0, strpos($class, '_'));
				$this->add_sub($type, $sub);
			}
		} else {
			$this->add_sub('summary', $options);
		}
	}

	/**
	 * @param $type string
	 * @param $options array|Report_options
	 */
	public function add_sub($type, $options)
	{
		if (isset($options['report_id']) && !empty($options['report_id']))
			$saved_options = Saved_reports_Model::get_report_info(self::$right_sub_type, $options['report_id']);
		else
			unset($options['report_id']);
		$sub = static::setup_options_obj($type, $options);
		if (isset($saved_options))
			static::merge_with_loaded($sub, $options, $saved_options);
		$this->options = array_merge($sub->options, $this->options);
		array_unshift($this->subs, $sub);
	}

	public function set($name, $value)
	{
		$res = parent::set($name, $value);
		foreach ($this->subs as $sub) {
			if (($res += $sub->set($name, $value))) {
				# handle weird side-effects
				$this->options = array_merge($this->options, $sub->options);
			}
		}
		return (bool)$res;
	}

	public function get_alternatives($key)
	{
		$out = parent::get_alternatives($key);
		if ($out !== false)
			return $out;
		$current = false;
		foreach ($this->subs as $sub) {
			$out = $sub->get_alternatives($key);
			if ($out !== false) {
				if($out) {
					// return the first non-falsy value
					return $out;
				}
				$current = $out; // might for example be an empty array or a '0'
			}
		}
		return $current;
	}

	public function offsetGet($str)
	{
		$out = parent::offsetGet($str);
		if ($out !== NULL)
			return $out;
		$current = NULL;
		foreach ($this->subs as $sub) {
			$out = $sub->offsetGet($str);
			if ($out !== NULL)
				return $out;
		}
	}

	public function get_value($key)
	{
		$out = parent::get_value($key);
		if ($out !== false)
			return $out;
		$current = false;
		foreach ($this->subs as $sub) {
			$out = $sub->get_value($key);
			if ($out !== false) {
				if($out) {
					// return the first non-falsy value
					return $out;
				}
				$current = $out; // might for example be an empty array or a '0'
			}
		}
		return $current;
	}

	function next() {
		while (true) {
			$x = next($this->options);
			if ($x === false)
				break;
			if (isset($this->properties[key($this->options)]) && !isset($this->properties[key($this->options)]['generated']))
				break;
			foreach ($this->subs as $sub) {
				if (isset($sub->properties[key($this->options)]) && !isset($sub->properties[key($this->options)]['generated']))
					break 2;
			}
		}
		return $x;
	}

	protected static function create_options_obj($type, $report_info = false)
	{
		return parent::create_options_obj(self::$right_sub_type, $report_info);
	}

	public function expand() {
		foreach ($this->properties as $key => $_) {
			$this[$key] = $this[$key];
		}
		foreach ($this->subs as $sub) {
			$sub->expand();
			$this->options = array_merge($sub->options, $this->options);
		}
	}
}
